from . import odoo
import logging
import os
import argparse
import configparser
import getpass
import csv
import pprint

LOGGER = logging.getLogger(__name__)

PP = pprint.PrettyPrinter(indent=4)

def pfdebug(logger, data):
    for line in PP.pformat(data).split('\n'):
        logger.debug(line)

class SimpleTool(object):
    def __init__(self):
        self._client = None
        self._config = None
        self._env = None
        self._askpassword = False
        self._auth = None

    def get_client(self):
        return self._client

    def load_config(self):
        config = configparser.ConfigParser()
        config.read([os.path.expanduser('~/.config/odoopti.cfg')])
        self._config = config

    def env_is_valid(self):
        return 'env:{}'.format(self._env) in self._config.sections()
        
    def env_sect(self):
        return self._config['env:{}'.format(self._env)]

    def config_env_get(self, key, default=None):
        return self.env_sect().get(key, default)

    def config_env_getint(self, key, default=0):
        return self.env_sect().getint(key, default)
        
    def config_get(self, section, key):
        return self._config.get(section, key)

    def config_getint(self, section, key):
        return self._config.getint(section, key)

    def connect(self):
        if not self.env_is_valid():
            raise Exception('Invalid environment: {}'.format(self._env))

        url = self.config_env_get('server_url', '')
        db = self.config_env_get('database')
        user = self.config_env_get('user')
        
        if not url:
            host = self.config_env_get('server_host')
            port = self.config_env_getint('server_port')

            proto = 'http'
            if port == 443:
                proto = 'https'
            url = f'{proto}://{host}:{port}'

        if not (url and db and user):
            raise Exception('Cannot get valid url/db/user from config file')
            
        
        LOGGER.info(f'Connecting to Odoo server {url} as user {user}')

        password = self.config_env_get('password')
        if self._askpassword:
            password = getpass.getpass()

        self._client = odoo.Odoo(url).login(db, user, password)

    def call(self, model, method, args, kwargs):
        return self._client.call(model, method, args, kwargs)

    def search(self, model, domain, context=None, pos='all'):
        if pos == 'all':
            return self._client.search(model, domain, context)
        elif pos == 'first':
            return self._client.search_first(model, domain, context)
        elif pos == 'one':
            return self._client.search_one(model, domain, context)

    def read(self, model, ids, fields, context=None):
        return self._client.read(model, ids, fields, context)

    def read_one(self, model, single_id, fields, context=None):
        return self._client.read_one(model, single_id, fields, context)

    def search_read(self, model, domain, fields, context=None, pos='all'):
        if pos == 'all':
            return self._client.search_read(model, domain, fields, context)
        elif pos == 'first':
            return self._client.search_read_first(model, domain, fields, context)
        elif pos == 'one':
            return self._client.search_read_one(model, domain, fields, context)

    def create(self, model, data, context=None):
        return self._client.create(model, data, context)

    def write(self, model, ids, data, context=None):
        return self._client.write(model, ids, data, context)

    def csv_read(self, filename):
        result = []
        try:
            with open(filename, 'rt') as f:
                reader = csv.DictReader(f)
                for row in reader:
                    result.append(row)
        except IOError:
            LOGGER.error('Could not read %s', filename)
        return result

    def boot_add_arguments(self, parser):
        parser.add_argument('-l', '--logging', choices=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'], help='Set log level', default='WARNING')
        parser.add_argument('-e', '--env', type=str, default='devel', help='Set environment')
        parser.add_argument('-w', '--readpassword', help='Prompt for password', action='store_true')

    def boot_process_arguments(self, args):
        if args.logging:
            lvl = getattr(logging, args.logging, None)
            if isinstance(lvl, int):
                logging.getLogger().setLevel(lvl)

        self._env = args.env
        self._askpassword = args.readpassword

    def bootstrap(self):
        self.load_config()
        logging.basicConfig(level=logging.INFO, format='%(asctime)s %(levelname)s: %(message)s')

        parser = argparse.ArgumentParser()
        self.boot_add_arguments(parser)

        args = parser.parse_args()
        
        self.boot_process_arguments(args)

